require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'
require 'csv'

module PragGame
	class Game
		attr_reader :title

		def initialize(title)
			@players = []
			@title = title.capitalize
		end

		def load_players(from_file)
			CSV.foreach(from_file) do |row|
				add_player(Player.new(row[0], row[1].to_i))
			end
		end

		def save_high_scores(to_file = "high_scores.txt")
			File.open(to_file, "w") do |file|
				file.puts "#{title} High Scores:"
				@players.sort.each do |player|
					file.puts high_score_entry(player)
				end
			end
		end

		def add_player(player)
			@players.push(player)
		end

		def play(rounds)
			puts "There are #{@players.size} players in the #{@title}:"
			@players.each do |player|
				puts player
			end

			treasures = TreasureTrove::TREASURES
			puts "\nThere are #{treasures.size} treasures to be found:"
			treasures.each do |treasure|
				puts "A #{treasure.name} is worth #{treasure.points} points."
			end

			1.upto(rounds) do |count|
				if block_given?
					break if yield
				end
				puts "\nRound #{count}:"
				@players.each do |player|			
					GameTurn.take_turn(player)
					puts player
				end
			end
		end

		def print_name_and_health(player)
			puts "#{player.name} (#{player.health})"
		end

		def high_score_entry(player)
			formatted_name = player.name.ljust(20, '.')
			"#{formatted_name} #{player.score}"
		end

		def total_points
			@players.reduce(0) { |sum, player| sum + player.points }
		end

		def print_stats
			strong_players, wimpy_players = @players.partition { |n| n.strong? }

			puts "\n#{@title}'s stats:"

			@players.sort.each do |player|
				puts "\n#{player.name}'s point totals:"
				player.each_found_treasure do |treasure|
					puts "#{treasure.points} total #{treasure.name} points"
				end
				puts "#{player.points} grand total points"
			end

			puts "\n#{strong_players.size} strong players:"
			strong_players.each do |player|
				print_name_and_health(player)
			end

			puts "\n#{wimpy_players.size} wimpy players:"
			wimpy_players.each do |player|
				print_name_and_health(player)
			end

			puts "\n#{@title} High Scores:"
			@players.sort.each do |player|
				puts high_score_entry(player)
			end

			puts "\nThe total treasure points for the game is #{total_points}." 
		end
	end
end

if __FILE__ == $0
	player1 = Player.new("moe", 45)
	comedians = Game.new("comedians")
	comedians.add_player(player1)
	puts comedians.inspect
end